package com.fs.store.service.impl;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.*;
import java.math.BigDecimal;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import cn.hutool.json.JSONUtil;
import com.fs.common.config.FSSysConfig;
import com.fs.common.core.domain.R;
import com.fs.common.exception.CustomException;
import com.fs.common.utils.DateUtils;
import com.fs.common.utils.OrderUtils;
import com.fs.common.utils.StringUtils;
import com.fs.store.bean.DrugV2;
import com.fs.store.bean.PrescribeV2;
import com.fs.store.domain.*;
import com.fs.store.dto.FsPrescribeUsageDTO;
import com.fs.store.mapper.FsInquiryOrderMapper;
import com.fs.store.mapper.FsPrescribeDrugMapper;
import com.fs.store.param.*;
import com.fs.store.service.*;
import com.fs.store.vo.FsInquiryOrderVO;
import com.fs.store.vo.FsPrescribeVO;
import com.fs.system.oss.CloudStorageService;
import com.fs.system.oss.OSSFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.fs.store.mapper.FsPrescribeMapper;

import javax.imageio.ImageIO;

/**
 * 处方Service业务层处理
 *
 * @author fs
 * @date 2022-03-15
 */
@Service
public class FsPrescribeServiceImpl implements IFsPrescribeService
{
    @Autowired
    private IPrescribeService prescribeService;
    @Autowired
    private FsPrescribeMapper fsPrescribeMapper;
    @Autowired
    private IFsStoreOrderService orderService;
    @Autowired
    private IFsPatientService patientService;
    @Autowired
    private IFsUserService userService;
    @Autowired
    private FSSysConfig fsSysConfig;
    @Autowired
    private IFsStoreOrderItemService orderItemService;
    @Autowired
    private IFsStoreProductService productService;

    @Autowired
    private FsInquiryOrderMapper fsInquiryOrderMapper;

    @Autowired
    private FsPrescribeDrugMapper fsPrescribeDrugMapper;
    /**
     * 查询处方
     *
     * @param prescribeId 处方ID
     * @return 处方
     */
    @Override
    public FsPrescribe selectFsPrescribeById(Long prescribeId)
    {
        return fsPrescribeMapper.selectFsPrescribeById(prescribeId);
    }

    /**
     * 查询处方列表
     *
     * @param fsPrescribe 处方
     * @return 处方
     */
    @Override
    public List<FsPrescribe> selectFsPrescribeList(FsPrescribe fsPrescribe)
    {
        return fsPrescribeMapper.selectFsPrescribeList(fsPrescribe);
    }

    /**
     * 新增处方
     *
     * @param fsPrescribe 处方
     * @return 结果
     */
    @Override
    public int insertFsPrescribe(FsPrescribe fsPrescribe)
    {
        fsPrescribe.setCreateTime(DateUtils.getNowDate());
        return fsPrescribeMapper.insertFsPrescribe(fsPrescribe);
    }

    /**
     * 修改处方
     *
     * @param fsPrescribe 处方
     * @return 结果
     */
    @Override
    public int updateFsPrescribe(FsPrescribe fsPrescribe)
    {
        return fsPrescribeMapper.updateFsPrescribe(fsPrescribe);
    }

    /**
     * 批量删除处方
     *
     * @param prescribeIds 需要删除的处方ID
     * @return 结果
     */
    @Override
    public int deleteFsPrescribeByIds(Long[] prescribeIds)
    {
        return fsPrescribeMapper.deleteFsPrescribeByIds(prescribeIds);
    }

    /**
     * 删除处方信息
     *
     * @param prescribeId 处方ID
     * @return 结果
     */
    @Override
    public int deleteFsPrescribeById(Long prescribeId)
    {
        return fsPrescribeMapper.deleteFsPrescribeById(prescribeId);
    }

    @Override
    public R doPrescribe(long userId, FsPrescribeParam param) {
        FsStoreOrder order=orderService.selectFsStoreOrderById(param.getOrderId());
        if(order==null){
            throw  new CustomException("订单不存在");
        }
//      if(order.getStatus()!=1){
//           throw  new CustomException("未支付订单不能开处方");
//      }
        FsPrescribe checkPrescribe=fsPrescribeMapper.selectFsPrescribeByOrderId(order.getId());
        if(checkPrescribe!=null){
            throw  new CustomException("已提交处方申请，正在开方...");
        }
        FsStoreOrderItem orderItemMap=new FsStoreOrderItem();
        orderItemMap.setOrderId(order.getId());
        orderItemMap.setIsPrescribe(1);
        List<FsStoreOrderItem> items= orderItemService.selectFsStoreOrderItemList(orderItemMap);
        String rpId=OrderUtils.getOrderNo();
        FsPatient patient=patientService.selectFsPatientById(param.getPatientId());
        FsUser user=userService.selectFsUserById(userId);
        PrescribeV2Param prescribeParam=new PrescribeV2Param();
        PrescribeV2 prescribe=new PrescribeV2();
        prescribe.setDrugType("01");
        prescribe.setOnlyId(rpId);
        prescribe.setDrugstoreName("零利润药房");
        prescribe.setPatientName(patient.getPatientName());
        prescribe.setPatientAge(DateUtils.getAge(patient.getBirthday()));
//        prescribe.setPatientTel(user.getPhone());
        prescribe.setPatientSex(patient.getGender());
        prescribe.setPatientAllergy(param.getIsAllergic()?1:0);
        prescribe.setLiverUnusual(param.getIsLiver()?1:0);
        prescribe.setRenalUnusual(param.getIsRenal()?1:0);
        prescribe.setLactationFlag(param.getIsLactation()?1:0);
        prescribe.setChiefComplaint(param.getChiefComplaint());
        prescribe.setNowIllness(param.getNowIllness());
        prescribe.setHistoryIllness(param.getHistoryIllness());
        if(StringUtils.isNotEmpty(param.getRecordPic())){
            prescribe.setRecordPic(param.getRecordPic().split(","));
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date now = new Date();
        prescribe.setCreateTime(sdf.format(now));
        prescribe.setPictureType("png");
//        prescribe.setLactationFlag("否");
        prescribe.setCallbackUrl(fsSysConfig.getCallbackUrl());

        List<DrugV2> drug_list=new ArrayList<>();
        for(FsStoreOrderItem item:items){
            FsStoreProduct product=productService.selectFsStoreProductById(item.getProductId());
            DrugV2 drug=new DrugV2();
            drug.setDrugCommonName(product.getPrescribeName());
            drug.setDrugSpecification(product.getPrescribeSpec());
            drug.setSaleAmount(item.getNum().toString());
            drug.setSaleUnit(product.getUnitName());
            drug.setApprovalNumber(product.getPrescribeCode());
            drug.setDrugCode(product.getProductId());
            drug_list.add(drug);
            prescribe.setDrugList(drug_list);
            prescribeParam.setData(prescribe);
        }
        R response=prescribeService.doPrescribeV2(prescribeParam);
//        if(response.get("code").equals(200)){
        FsPrescribe fsPrescribe=new FsPrescribe();
        fsPrescribe.setRpId(rpId);
        fsPrescribe.setUserId(userId);
        fsPrescribe.setPatientId(patient.getPatientId());
        fsPrescribe.setPharmacyCode("00001");
        fsPrescribe.setPharmacyName("零利润药房");
        fsPrescribe.setStatus(0);
        fsPrescribe.setPrescribeType(1);
        fsPrescribe.setStoreOrderId(order.getId());
        fsPrescribe.setCreateTime(new Date());
        fsPrescribe.setPatientGender(patient.getGender().toString());
        fsPrescribe.setPatientName(patient.getPatientName());
        fsPrescribe.setIsHistoryAllergic(param.getIsAllergic()?"是":"否");
        fsPrescribe.setLiverUnusual(param.getIsLiver()?"是":"否");
        fsPrescribe.setLactationFlag(param.getIsLactation()?"是":"否");
        fsPrescribe.setRenalUnusual(param.getIsRenal()?"是":"否");
        fsPrescribe.setChiefComplaint(param.getChiefComplaint());
        fsPrescribe.setNowIllness(param.getNowIllness());
        fsPrescribe.setHistoryIllness(param.getHistoryIllness());
        fsPrescribe.setRecordPic(param.getRecordPic());
        fsPrescribe.setPatientAge(String.valueOf( DateUtils.getAge(patient.getBirthday())));
        fsPrescribeMapper.insertFsPrescribe(fsPrescribe);

        FsStoreOrder orderMap=new FsStoreOrder();
        orderMap.setId(order.getId());
        orderMap.setPrescribeId(fsPrescribe.getPrescribeId());
        orderService.updateFsStoreOrder(orderMap);
        return R.ok("操作成功").put("order",order);
//        }
//        else{
//            return R.error(response.get("msg").toString());
//        }
    }

    @Override
    public FsPrescribe selectFsPrescribeByRpId(String rpId) {
        return fsPrescribeMapper.selectFsPrescribeByRpId(rpId);
    }

    @Override
    public List<FsPrescribeVO> selectFsPrescribeListQuery(FsPrescribeQueryParam param) {
        return fsPrescribeMapper.selectFsPrescribeListQuery(param);

    }

    @Override
    public List<FsPrescribeVO> selectFsPrescribeListVO(FsPrescribeVOParam fsPrescribe) {
        return fsPrescribeMapper.selectFsPrescribeListVO(fsPrescribe);
    }

    @Override
    public FsPrescribe selectFsPrescribeByOrderId(Long id) {
        return fsPrescribeMapper.selectFsPrescribeByOrderId(id);
    }

    @Override
    public String PrescribeImg(Long prescribeId) {
        FsPrescribeVO f = fsPrescribeMapper.selectFsPrescribeByPrescribeIdVO(prescribeId);
        if (f==null){
            return "";
        }
        FsStoreOrder fsStoreOrder = orderService.selectFsStoreOrderById(f.getStoreOrderId());
        FsInquiryOrderVO order = fsInquiryOrderMapper.selectFsInquiryOrderVOByOrderId(f.getInquiryOrderId());
        if (f.getPrescribeImgUrl()==null||f.getPrescribeImgUrl().equals("")){
            FsPrescribeDrug d = new FsPrescribeDrug();
            d.setPrescribeId(f.getPrescribeId());
            List<FsPrescribeDrug> list = fsPrescribeDrugMapper.selectFsPrescribeDrugList(d);
//            List<FsPrescribeDrug> list =new ArrayList<>();
//            for (FsPrescribeDrug fsPrescribeDrug : FsPrescribeDruglist) {
//                if (fsPrescribeDrug.getIsDrug()!=null&&fsPrescribeDrug.getIsDrug()!=0){
//                    list.add(fsPrescribeDrug);
//                }else if (fsPrescribeDrug.getIsDrug()==null){
//                    list.add(fsPrescribeDrug);
//                }
//            }
            if (list.size()==0){
                return "";
            }
            if (f.getPrescribeType()==1){
                PrescribeXyImgParam o = new PrescribeXyImgParam();
                o.setTotalPrice(fsStoreOrder.getPayPrice());
                o.setTime(f.getCreateTime());
                o.setPrescribeId(f.getPrescribeCode());
                o.setPatientName(f.getPatientName());
                o.setPatientGender(f.getPatientGender());
                o.setPatientAge(f.getPatientAge());
                o.setOutpatientId(f.getPrescribeCode());
                o.setUrl(f.getPrescribeDoctorSignUrl());
                o.setDrugDoctorUrl(f.getDrugDoctorSignUrl());
                if (order!=null){
                    o.setBedId(order.getDeptName());
                }else {
                    o.setBedId("中医科");
                }
                String remark="";
                if (f.getUsageJson()!=null){
                    FsPrescribeUsageDTO usage = JSONUtil.toBean(f.getUsageJson(), FsPrescribeUsageDTO.class);
                    remark=usage.getRemark();
                }
                if (remark!=null&&remark!=""){
                    o.setRemark(remark);
                }else {
                    o.setRemark("请按照用药说明书服用药品，如有不适，请及时就医!");
                }
                o.setHistoryAllergic(f.getHistoryAllergic());
                o.setDiagnose(f.getDiagnose());
                o.setPrescribeDrug(list); // 在这里添加处方药品列表的值
                o.setDoctorName(f.getDoctorName());
                o.setAuditDoctor(f.getDoctorDrugName());
                o.setDispatcher("");
                o.setCheckDoctor("");
                o.setDoctorName(f.getDoctorName());
                String s = getFsPrescribeXyImg(o);
                FsPrescribe fsPrescribe = new FsPrescribe();
                fsPrescribe.setRpUrl(s);
                fsPrescribe.setPrescribeId(f.getPrescribeId());
                fsPrescribeMapper.updateFsPrescribe(fsPrescribe);
                return s;
            }else if (f.getPrescribeType()==2){

                FsPrescribeUsageDTO usage = JSONUtil.toBean(f.getUsageJson(), FsPrescribeUsageDTO.class);

                // 设置字段值
                PrescribeZyImgParam o = new PrescribeZyImgParam();
                o.setTime(f.getCreateTime());

                o.setPrescribeId(f.getPrescribeCode());
                o.setPatientName(f.getPatientName());
                o.setPatientGender(f.getPatientGender());
                o.setPatientAge(f.getPatientAge());
                o.setPatientTel(f.getPatientTel());
                o.setPrescribeDoctorName(f.getPrescribeDoctorName());
                o.setUserName(fsStoreOrder.getRealName()); //发货地址
                o.setUserPhone(fsStoreOrder.getUserPhone()); //收货人手机
                o.setUserAddress(fsStoreOrder.getUserAddress()); //收货地址
                o.setDiagnose(f.getDiagnose());
                o.setPrescribeDrug(list); // 在这里添加处方药品列表的值
                o.setEat(usage.getUsageMethod());
                o.setUsageFrequencyUnit(usage.getUsageFrequencyUnit());
                o.setRemark(usage.getRemark());
                o.setDoctorName(f.getDoctorName());
                o.setCheck(f.getDoctorDrugName());
                if (usage.getCounts()!=null){
                    o.setSize(usage.getCounts().toString());
                }
                o.setUrl(f.getPrescribeDoctorSignUrl());
                o.setDrugDoctorUrl(f.getDrugDoctorSignUrl());
                o.setTotalPrice(fsStoreOrder.getPayPrice());
                String s = getFsPrescribeZyImg(o);
                FsPrescribe fsPrescribe = new FsPrescribe();
                fsPrescribe.setRpUrl(s);
                fsPrescribe.setPrescribeId(f.getPrescribeId());
                fsPrescribeMapper.updateFsPrescribe(fsPrescribe);
                return s;
            }
        }
        return f.getPrescribeImgUrl();
    }


    @Override
    public String getFsPrescribeZyImg(PrescribeZyImgParam o) {
        File mb = new File("C:\\fs\\zycf.jpg");
        if (!mb.exists()) {
            // 创建目录
            throw  new CustomException("模板文件不存在");
        }
        BufferedImage image = null;
        try {
            image = ImageIO.read(mb);
        } catch (IOException e) {
            e.printStackTrace();
        }
        // 创建画笔(image为上一步的图片对象)
        Graphics2D pen = image.createGraphics();
        pen.setColor(Color.black);
        //处理字体 高清加边缘模糊
        pen.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
                RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        pen.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        pen.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
        pen.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_SPEED);
        pen.setFont(new Font("黑体", Font.PLAIN, 21));
        // 将date对象格式化为字符串

        penDrawString(pen,new SimpleDateFormat("yyyy-MM-dd").format(o.getTime()), 730, 290);
        penDrawString(pen,o.getPrescribeId(),1030,290);
        penDrawString(pen,o.getPatientName(),136,372);
        penDrawString(pen,o.getPatientGender().equals("1")?"男":o.getPatientGender().equals("2")?"女":o.getPatientGender(),420,372);
        penDrawString(pen,o.getPatientAge(),690,372);
        penDrawString(pen,o.getPatientTel(),970,372);
        String address= o.getUserName()+"      "+o.getUserPhone()+"     "+o.getUserAddress();
        penDrawString(pen,address,180,445);
        penDrawString(pen,o.getDiagnose(),180,515);
        List<FsPrescribeDrug> Drug = o.getPrescribeDrug();
        int size=Drug.size();

        for (int i = 0; i < size/3.0; i++) {
            if (i*3<size){
                pen.drawString(Drug.get(i*3).getDrugName(),145,680+i*50);
            }
            if (i*3+1<size){
                pen.drawString(Drug.get(i*3+1).getDrugName(),510,680+i*50);
            }
            if (i*3+2<size){
                pen.drawString(Drug.get(i*3+2).getDrugName(),875,680+i*50);
            }
        }
        penDrawString(pen,o.getEat(),110,1195);
        penDrawString(pen,o.getUsageFrequencyUnit(),135,1240);
        penDrawString(pen,o.getRemark(),110,1288);

        // penDrawString(pen,o.getDoctorName(),152,1378);
        if (o.getUrl()!=null){
            try {
                URL imageUrl = new URL(o.getUrl());
                BufferedImage img = ImageIO.read(imageUrl);
                pen.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
                pen.drawImage(img, 130, 1420, 127, 60, null);
            } catch (Exception e) {
                throw new CustomException("获取医生签名图片失败");
            }

        }else {
            penDrawString(pen,o.getPrescribeDoctorName(),152,1450);
        }
        if (o.getDrugDoctorUrl()!=null){
            try {
                URL   imageUrl3 = new URL(o.getDrugDoctorUrl());
                BufferedImage img3 = ImageIO.read(imageUrl3);
                pen.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
                pen.drawImage(img3, 370, 1345, 127, 60, null);
            } catch (Exception e) {
                throw new CustomException("获取药师签名图片失败");
            }
        }
//        penDrawString(pen,o.getCheck(),348,1378);
        penDrawString(pen,o.getSize(),620,1378);
        if (o.getTotalPrice()!=null){
            penDrawString(pen,o.getTotalPrice().toString(),370,1452);
            penDrawString(pen,o.getTotalPrice().toString(),620,1452);
        }
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        try {
            ImageIO.write(image, "png", os);
        } catch (IOException e) {
            e.printStackTrace();
        }
        InputStream inputStream = new ByteArrayInputStream(os.toByteArray());
        CloudStorageService storage = OSSFactory.build();
        String url = storage.uploadSuffix(inputStream, ".jpg");
        return url;
    }

    @Override
    public String getFsPrescribeXyImg(PrescribeXyImgParam o) {
        // File mb = new File(System.getProperty("user.dir")+"\\xycf.jpg");
        File mb = new File("C:\\fs\\xycf.jpg");
        if (!mb.exists()) {
            // 创建目录
            throw  new CustomException("模板文件不存在");
        }
        BufferedImage image = null;
        try {
            image = ImageIO.read(mb);
        } catch (IOException e) {
            e.printStackTrace();
        }
        // 创建画笔(image为上一步的图片对象)
        Graphics2D pen = image.createGraphics();
        pen.setColor(Color.black);
        //处理字体 高清加边缘模糊
        pen.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
                RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        pen.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        pen.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
        pen.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_SPEED);
        pen.setFont(new Font("黑体", Font.PLAIN, 14));
        penDrawString(pen,o.getPrescribeId(),448,145);
        penDrawString(pen,o.getPatientName(),60,178);
        if (o.getPatientGender().equals("1")){
            penDrawString(pen,"√",255,178);
        }else if(o.getPatientGender().equals("2")){
            penDrawString(pen,"√",298,178);
        }
        penDrawString(pen,o.getPatientAge(),400,178);
        penDrawString(pen,o.getOutpatientId(),92,205);
        penDrawString(pen,o.getBedId(),400,205);
        penDrawString(pen,o.getHistoryAllergic(),95,230);
        penDrawString(pen,o.getDiagnose(),95,257);
        penDrawString(pen,new SimpleDateFormat("yyyy-MM-dd").format(o.getTime()),400,257);
        List<FsPrescribeDrug> Drug = o.getPrescribeDrug();
        int size=Drug.size();
//        BigDecimal price=BigDecimal.valueOf(0);
        for (int i = 0; i < size; i++) {
            FsPrescribeDrug d = Drug.get(i);
            String drugName = d.getDrugName();
            BigDecimal drugPrice = d.getDrugPrice().multiply(BigDecimal.valueOf(d.getDrugNum()));
//            price= price.add(drugPrice);
            if (drugName.length()>30){
                drugName = drugName.substring(0, 30)+"...";
            }

            String drugSpec = (d.getDrugSpec() != null) ? d.getDrugSpec() : "   ";

            penDrawString(pen,i+1+","+drugName+"    "+drugSpec+"    "+d.getDrugNum(),35,305+i*40);
            String usageMethod = (d.getUsageMethod() != null) ? d.getUsageMethod() : "   ";
            String usageFrequencyUnit = (d.getUsageFrequencyUnit() != null) ? d.getUsageFrequencyUnit() : "   ";
            String usagePerUseCount = (d.getUsagePerUseCount() != null) ? d.getUsagePerUseCount() : "   ";

            penDrawString(pen," Sig:          "+usageMethod+"     "+usageFrequencyUnit+"     "+usagePerUseCount,35,325+i*40);
        }
        penDrawString(pen,"本页完",277,325+size*40);

        penDrawString(pen,o.getRemark(),62,651);
        if (o.getUrl()!=null){
            try {
                URL   imageUrl = new URL(o.getUrl());
                pen.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
                BufferedImage img = ImageIO.read(imageUrl);
                pen.drawImage(img, 65, 688, 88, 40, null);
            } catch (Exception e) {
                throw new CustomException("获取签名图片失败");
            }
        }else {
            penDrawString(pen,o.getDoctorName(),63,716);
        }

        penDrawString(pen,o.getTotalPrice().toString(),375,717);
        if (o.getDrugDoctorUrl()!=null){
            try {
                URL   imageUrl = new URL(o.getDrugDoctorUrl());
                BufferedImage img = ImageIO.read(imageUrl);
                pen.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
                pen.drawImage(img, 90, 730, 88, 40, null);
            } catch (Exception e) {
                throw new CustomException("获取药师签名图片失败");
            }
        }
        penDrawString(pen,o.getDispatcher(),357,757);
        penDrawString(pen,o.getCheckDoctor(),135,796);


        ByteArrayOutputStream os = new ByteArrayOutputStream();
        try {
            ImageIO.write(image, "png", os);
        } catch (IOException e) {
            e.printStackTrace();
        }
        InputStream inputStream = new ByteArrayInputStream(os.toByteArray());
        CloudStorageService storage = OSSFactory.build();
        String url = storage.uploadSuffix(inputStream, ".jpg");
        return url;
    }

    public void penDrawString( Graphics2D pen,String value,int x,int y){
        if (value!=null){
            pen.drawString(value,x,y);
        }
    }
}
